@@ -1,29 +1,30 @@
-import json
+import yaml, json, random
 import socket, select
+
 from . import logger
 from . import util
 from .com import Connection, Rule
-from .msg import *
-import random
+from .msg import Msg, MsgType
 
 
 class Config:
     def __init__(self, config_file: str = None):
         self.addr = "0.0.0.0"
         self.port = 9001
-        self.max_num = 1024
-        self.rule_file = ''
+        self.rules = []
 
         if config_file:
             with open(config_file) as fp:
-                cfg = json.load(fp)
+                cfg = yaml.safe_load(fp)
                 self.addr = cfg['addr']
                 self.port = cfg['port']
-                self.max_num = cfg['max_num']
-                self.rule_file = cfg['rule_file']
+                self.rules = cfg['rules']
 
 
 class Server:
+    BUF_SIZE = 1024 * 256
+    MAX_NUM = 1024 * 4
+
     def __init__(self, config_file: str = None):
         self.config = Config(config_file)
         self.rules = {}
@@ -32,7 +33,7 @@ class Server:
         self.socket.bind((self.config.addr, self.config.port))
         self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, True)
         self.socket.setblocking(False)
-        self.socket.listen(self.config.max_num)
+        self.socket.listen(Server.MAX_NUM)
 
         self.outter_sockets = {}
 
@@ -46,12 +47,15 @@ class Server:
 
         self.rsockets, self.wsockets, self.xsockets = [self.socket], [], []
 
-        self.load_rules(self.config.rule_file)
+        for line in self.config.rules:
+            self.add_rule(line)
 
         logger.info("laurel start {}".format(json.dumps(self.config.__dict__)))
 
     def choose_inner_conn(self):
         cs = list(self.inner_conns.keys())
+        if len(cs) == 0:
+            return None
         return random.choice(cs)
 
     def create_outter_socket(self, rule: Rule):
@@ -61,7 +65,7 @@ class Server:
                 sk.bind((rule.to_addr, rule.to_port))
                 sk.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, True)
                 sk.setblocking(False)
-                sk.listen(self.config.max_num)
+                sk.listen(Server.MAX_NUM)
                 self.outter_sockets[sk] = rule
                 self.rsockets.append(sk)
                 logger.info("[SERVER] create rule {}".format(rule))
@@ -90,12 +94,16 @@ class Server:
 
     def open_outter_conn(self, outter_id: str, conn: socket):
         if outter_id not in self.outter_conns:
+            inner_id = self.choose_inner_conn()
+            if inner_id is None:
+                logger.error("[SERVER] no client for outter conn {}".format(outter_id))
+                return
+
             self.outter_conns[outter_id] = Connection(outter_id, conn)
             self.outter_ids[conn] = outter_id
             conn.setblocking(False)
             self.rsockets.append(conn)
             self.wsockets.append(conn)
-            inner_id = self.choose_inner_conn()
             self.outter_id_to_inner_id[outter_id] = inner_id
             logger.info("[SERVER] open outter conn {}".format(outter_id))
 
@@ -109,42 +117,41 @@ class Server:
             del self.outter_ids[connection.conn]
             logger.info("[SERVER] close outter conn {}".format(outter_id))
 
-    def load_rules(self, file: str):
-        with open(file, 'r') as fp:
-            for line in fp.readlines():
-                line = line.strip()
-                rule = Rule()
-                if rule.parse_string(line) == 0:
-                    self.create_outter_socket(rule)
+    def add_rule(self, line: str):
+        line = line.strip()
+        rule = Rule()
+        if rule.parse_string(line) == 0:
+            self.create_outter_socket(rule)
 
     def read_handler(self, rsockets: list):
         for rsocket in rsockets:
             if rsocket is self.socket:
                 conn, addr = rsocket.accept()
-                inner_id = "{}:{}".format(addr[0], addr[1])
+                inner_id = "{},{}".format(addr[0], addr[1])
                 self.open_inner_conn(inner_id, conn)
 
             elif rsocket in self.outter_sockets:
-                conn, addr = rsocket.accept()
-                outter_id = "{}:{}".format(addr[0], addr[1])
-                self.open_outter_conn(outter_id, conn)
-
                 rule = self.outter_sockets[rsocket]
 
-                inner_id = self.outter_id_to_inner_id[outter_id]
-                inner_conn = self.inner_conns[inner_id]
-                msg = Msg(MsgType.OPEN_CONN, outter_id, )
-                util.push_msg(inner_conn.input_buf, MsgType.OPEN_CONN, outter_id, bytes(rule.__str__(), "utf-8"))
+                conn, addr = rsocket.accept()
+                outter_id = "{},{},{},{}".format(rule.from_addr, rule.from_port, addr[0], addr[1])
+                self.open_outter_conn(outter_id, conn)
 
-                if inner_conn.conn not in self.wsockets:
-                    self.wsockets.append(inner_conn.conn)
+                if outter_id in self.outter_id_to_inner_id:
+                    inner_id = self.outter_id_to_inner_id[outter_id]
+                    inner_conn = self.inner_conns[inner_id]
+                    msg = Msg(MsgType.OPEN_CONN, outter_id, )
+                    util.push_msg(inner_conn.input_buf, MsgType.OPEN_CONN, outter_id, bytes(rule.__str__(), "utf-8"))
+
+                    if inner_conn.conn not in self.wsockets:
+                        self.wsockets.append(inner_conn.conn)
 
             elif rsocket in self.inner_ids:
                 inner_id = self.inner_ids[rsocket]
                 inner_conn = self.inner_conns[inner_id]
 
                 try:
-                    data = rsocket.recv(1024 * 128)
+                    data = rsocket.recv(Server.BUF_SIZE)
 
                     logger.debug("[SERVER] recv from inner {}".format(len(data)))
 
@@ -173,7 +180,7 @@ class Server:
             elif rsocket in self.outter_ids:
                 outter_id = self.outter_ids[rsocket]
                 try:
-                    data = rsocket.recv(1024 * 128)
+                    data = rsocket.recv(Server.BUF_SIZE)
                     if len(data) > 0:
                         if outter_id not in self.outter_id_to_inner_id:
                             raise ValueError("can't find inner id")
@@ -198,7 +205,7 @@ class Server:
                     self.rsockets.remove(rsocket)
 
     def write_handler(self, wsockets: list):
-        for wsocket in self.wsockets:
+        for wsocket in wsockets:
             if wsocket in self.inner_ids:
                 inner_id = self.inner_ids[wsocket]
                 inner_conn = self.inner_conns[inner_id]
@@ -251,12 +258,12 @@ class Server:
 
     def start(self):
         while True:
-            rsockets, wsockets, xsockets = select.select(self.rsockets, self.wsockets, self.xsockets, 1000)
+            rsockets, wsockets, xsockets = select.select(self.rsockets, self.wsockets, self.xsockets, 1)
             self.read_handler(rsockets)
             self.write_handler(wsockets)
             self.exception_handler(xsockets)
 
 
 if __name__ == '__main__':
-    ms = Server('cfg.json')
+    ms = Server('config.json')
     ms.start()
